/*
 * faces.h
 *
 * definition of the faces used by a2ps
 * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
 * Copyright (c) 1995, 96, 97 Akim Demaille, Miguel Santana
 * $Id: faces.c,v 1.25 1997/09/02 15:38:42 demaille Exp $
 */

/*
 * This file is part of a2ps.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "a2ps.h"
#include "faces.h"
#include "routines.h"
#include "jobs.h"

struct face_and_name {
  const char * name;
  face_t face;
};

static const struct face_and_name face_and_name [] = {
  { "NoFace", No_face },
  { "Plain", Plain },
  { "Symbol", Symbol },
  { "Keyword", Keyword },
  { "Keyword_strong", Keyword_strong },
  { "Comment", Comment },
  { "Comment_strong", Comment_strong },
  { "Label", Label },
  { "Label_strong", Label_strong },
  { "String", String },
  { "Tag1", TAG1 },
  { "Tag2", TAG2 },
  { "Tag3", TAG3 },
  { "Tag4", TAG4 },
  { "Error", Error},
  { "Index1", Index1 },
  { "Index2", Index2 },
  { "Index3", Index3 },
  { "Index4", Index4 },
  { "Encoding", ENCODING_TAG },
  { NULL, 0 }
};

face_t
string_to_face (const char * string)
{
  int i;
  
  for (i = 0 ; face_and_name[i].name ; i++)
    if (strequ (string, face_and_name[i].name))
      return face_and_name[i].face;
  
  return No_face;
}

const char *
face_to_string (face_t face)
{
  int i;
  
  for (i = 0 ; face_and_name[i].name ; i++)
    if (face == face_and_name[i].face)
      return face_and_name[i].name;
  
  return "UnknownFace";
}

uchar *
xface_to_string (face_t face)
{
  uchar * res;

  if (face < FACE_MASK)
    return xustrdup (face_to_string (face));

  if (face == INVISIBLE)
    return xustrdup ("Invisible");

  res = ALLOC (uchar, 1000);

  if (face & INVISIBLE) {
    sprintf ((char *) res, "(Invisible");
    face &= ~INVISIBLE;
  }
  else 
    sprintf ((char *) res, "(%s", face_to_string (face & FACE_MASK));
  face &= ~ FACE_MASK;
  while (face) {
    ustrcat (res, " + ");
    if (face & TAG1) {
      face &= ~TAG1;
      ustrcat (res, "Tag1");
    }
    else if (face & TAG2) {
      face &= ~TAG2;
      ustrcat (res, "Tag2");
    }
    else if (face & TAG3) {
      face &= ~TAG3;
      ustrcat (res, "Tag3");
    }
    else if (face & TAG4) {
      face &= ~TAG4;
      ustrcat (res, "Tag4");
    }
    else if (face & Index1) {
      face &= ~Index1;
      ustrcat (res, "Index1");
    }
    else if (face & Index2) {
      face &= ~Index2;
      ustrcat (res, "Index2");
    }
    else if (face & Index3) {
      face &= ~Index3;
      ustrcat (res, "Index3");
    }
    else if (face & Index4) {
      face &= ~Index4;
      ustrcat (res, "Index4");
    }
    else if (face & ENCODING_TAG) {
      face &= ~ENCODING_TAG;
      ustrcat (res, "Encoding");
    }
  }

  ustrcat (res, ")");
  res = REALLOC (res, uchar, ustrlen (res) + 1);
  return res;
}

/* convertion from a font to the postscript function */
const char *
face_to_ps (face_t face)
{
  switch (face) {
  case Plain:
    return "p";
  case Symbol:
    return "sy";
  case Keyword:
    return "k";
  case Keyword_strong:
    return "K";
  case Comment:
    return "c";
  case Comment_strong:
    return "C";
  case Label:
    return "l";
  case Label_strong:
    return "L";
  case String:
    return "str";
  case Error:
    return "e";
  default:
    error (1, 0, "face_to_ps: %d", face);
  }
  return 0; /* For -Wall */
}

/* Check that the face_to_font knowldge is complete */
void
check_face_to_font (struct a2ps_job * job)
{
  face_t f;
  int err = FALSE;

  for (f = 0 ; f < NB_FACES ; f++)
    if (!job->face_to_font [f]) {
      err = TRUE;
      error (0, 0, "face `%s' has no specified font", face_to_string (f));
    }
  if (err)
    error (1, 0, _("incomplete knowledge of faces"));
}

/*
 * Return TRUE if the faces are known, FALSE otherwise
 */
int
face_to_font_is_set (struct a2ps_job * job)
{
  face_t f;
  int res = TRUE;

  for (f = 0 ; f < NB_FACES ; f++)
    if (!job->face_to_font [f])
      res = FALSE;

  return res;
}

/* Initialize the face_font_to_font array to non acceptable values */
void
init_face_to_font (struct a2ps_job * job)
{
  face_t f;
  for (f = 0 ; f < NB_FACES ; f++)
    job->face_to_font [f] = NULL;
}

void
face_to_font_free (char * face_to_font [NB_FACES])
{
  face_t f;
  for (f = 0 ; f < NB_FACES ; f++)
    /* Xfree because it may not have been initialized if no printing
     * was done (e.g. --help) */
    XFREE (face_to_font [f]);
}

void
set_face_font (struct a2ps_job * job, face_t face, const char * font_name)
{
  XFREE (job->face_to_font [face]);
  job->face_to_font [face] = xstrdup (font_name);
}
