/*
 * printers.c
 *
 * Recording information about the printers (named outputs)
 * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
 * Copyright (c) 1995, 96, 97 Akim Demaille, Miguel Santana
 */

/*
 * This file is part of a2ps.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * $Id: printers.c,v 1.31 1997/11/07 09:56:18 demaille Exp $
 */
#include "a2ps.h"
#include "jobs.h"
#include "printers.h"
#include "routines.h"
#include "xstrrpl.h"
#include "pair_ht.h"
#include "printers.h"
#include "msg.h"
#include "metaseq.h"
#include "stream.h"

/************************************************************************
 * hash tables for the printers						*
 ************************************************************************/
/*
 * Create the structure that stores the list of printers
 */
struct pair_htable *
new_printers_table (void)
{
  return pair_table_new ();
}

/*
 * Free the whole structure
 */
void
free_printers_table (struct pair_htable * table)
{
  pair_table_free (table);
}

/*
 *  Add a printer, with your own allocation for them.
 * Return TRUE iff success (i.e. correct command)
 */
int
add_printer (a2ps_job * job, const char * name, const char * command)
{
  /* The command must either starts with a `>' or a `|' */
  if ((*command != '>') && (*command != '|'))
    return FALSE;

  pair_add (job->printers, name, command);
  return TRUE;
}

/*
 * Retrieve command associated with a printer, (or default printer
 * if no printer given) and if SUBSTITUTE then perform
 * all the needed substitutions
 * The result is no longer malloc'ed
 */
static char *
get_printer_command (a2ps_job * job, 
		     const char * printer_name,
		     int substitute)
{
  char * command = NULL;

  /*
   * Get the command, before substitution
   */
  switch (IS_EMPTY (printer_name)) {
  case FALSE:	/* A printer name is given: this is -P<printer> */
    command = pair_get (job->printers, printer_name);
    
    if (!command) {
      /* The name of a printer is given, but it is not known */
      command = pair_get (job->printers, _("Unknown Printer"));
      if (!command)
	error (1, 0, _("no default command for unknown printer `%s'"), 
	       printer_name);
    }
    break;
    
  case TRUE:	/* No name of printer is given, this is -d */
    command = pair_get (job->printers, _("Default Printer"));
    if (!command)
      error (1, 0, _("no default command for option `-d'"));
    break;
  }

  if (substitute)
    /* Perform a substitution of %s in command */
    /* We give the first file, so that it may give its name to the
     * produced file */
    return (char *) expand_user_string (job, FIRST_FILE (job),
					(uchar *) _("output command"), 
					(uchar *) command);
  else
    return command;
}

/*
 * Format the presentation of a printer and its command for
 * --list-printers.
 */
static void
dump_printer_command_long (int i,
			   const char * name, const char * command,
			   FILE * stream)
{
  switch (*command) {
  case '|':
    fprintf (stream, _("  %-10s = pipe in %s\n"), name,
	     command + strspn (command, "\t |"));
    break;
  case '>':
    fprintf (stream, _("  %-10s = save in %s\n"), name, 
	     command + strspn (command, "\t >"));
    break;
  default:
    error (1, 0, "dump_printer_command");
  }
}

/*
 * Format the presentation of a printer and its command for
 * --list-printers.
 */
static void
dump_printer_command_short (int i,
			    const char * name, const char * command,
			    FILE * stream)
{
  if (!(i % 7))
    fputs ("\n  ", stream);
  fprintf (stream, "%-10s", name);
}

/*
 * TRUE iff not the default or unknwon printer
 */
static int
printer_is_named (const char * name, const char * command)
{
  return (!strequ (name, _("Default Printer"))
	  && !strequ (name, _("Unknown Printer")));
}

/*
 * For --list-printers
 */
void
list_printers_long (a2ps_job * job)
{
  fputs (_("Known output destination (printers, etc.)"), stdout);
  putc ('\n', stdout);

  pair_table_map (job->printers, 
			  (pair_ht_map_fn_t) dump_printer_command_long,
			  NULL,
			  (void const *) stdout);
}

/*
 * For --list-features
 * This one is hard to write: we don't want to see the
 * default and unknwon printers.
 */
void
list_printers_short (a2ps_job * job)
{
  printf (_("Known output destination (printers, etc.)"));
  pair_table_map (job->printers, 
			  (pair_ht_map_fn_t) dump_printer_command_short,
			  (pair_ht_select_fn_t) printer_is_named,
			  (void const *) stdout);
  putc ('\n', stdout);
}

/*
 * Open a stream on the specified output in JOB
 */
void
a2ps_open_output_stream (struct a2ps_job * job)
{
  /* Open the destination */
  if (job->lpr_print) {
    char * command;
    command = get_printer_command (job, job->output_name, TRUE);
    job->output_stream = stream_perl_open (command);
  } else {
    /* Back up the file before */
    if (! IS_EMPTY (job->output_name))
      xbackup_file (job->output_name);
    job->output_stream = stream_wopen (job->output_name, FALSE);
  }
}

/*
 * Close the output stream with fclose or pclose
 */
void
a2ps_close_output_stream (struct a2ps_job * job)
{
  stream_close (job->output_stream);
}

/*
 * Send a file as it is to stdout.
 * This is used for pass_trough files (e.g. PS files)
 */
void
file_into_stdout (const void * arg)
{
  const char * filename = (const char *) arg;
  FILE * fp;

  message (msg_file1, "Dumping `%s' onto stdout.\n", filename);
  fp = xrfopen (filename);
  streams_copy (fp, stdout);
  fclose (fp);
}

/*
 * Make a standard message upon the destination
 */
static void
dest_to_string (const char * name, int is_printer, uchar * buf)
{
  if (is_printer) {
    /* NAME is about a printer */
    if (!IS_EMPTY (name))
      sprintf ((char *) buf, _("sent to the printer `%s'"), name);
    else
      sprintf ((char *) buf, _("sent to the default printer"));
  } else {
    if (!IS_EMPTY (name))
      sprintf ((char *) buf, _("saved into the file `%s'"), name);
    else
      sprintf ((char *) buf, _("sent to the standard output"));
  }
}

/*
 * Return a string that describes the destination of the
 * file.
 */
void
destination_to_string (a2ps_job * job, uchar * buf)
{
  /* Make a nice message to tell where the output is sent */
  if (job->lpr_print)
    dest_to_string (job->output_name, 1, buf);
  else
    dest_to_string (job->output_name, 0, buf);
}

/*
 * The same, but after evaluation
 */
void
final_destination_to_string (a2ps_job * job, uchar * buf)
{
  /* The main difference is when sending to a file, in which
   * case we want to have its real name */
  char * lp_command = NULL;
  char * command;
  
  if (job->lpr_print) {
    lp_command = get_printer_command (job, job->output_name, TRUE);
    command = lp_command + strspn (lp_command, "\t >|");
    
    switch (*lp_command) {
    case '|':	/* Pipe, just give the symbolic name */
      dest_to_string (job->output_name, 1, buf);
      break;
      
    case '>':	/* File, give its real name */
      dest_to_string (command, 0, buf);
      break;
    }
  } else {
    dest_to_string (job->output_name, 0, buf) ;
  }
}
